@@ -7,4 +7,8 @@
 
 .onAttach <- function(libname, pkgname) {
   shiny::addResourcePath("sbs", system.file("www", package = "shinyBS"))
+}
+
+.onUnload <- function (libpath) {
+  library.dynam.unload("dexter", libpath)
 }
\ No newline at end of file
